package utils

import (
	"encoding/json"
	"fmt"
	"testing"

	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
)

// JSONEncodeBytes encodes the given byte slice to a JSON byte slice.
func JSONEncodeBytes(b []byte) (*apiextensionsv1.JSON, error) {
	data, err := json.Marshal(string(b))
	if err != nil {
		return nil, fmt.Errorf("could not encode bytes to json: %w", err)
	}
	return &apiextensionsv1.JSON{Raw: data}, nil
}

// MustJSONEncodeBytes is like JSONEncodeBytes but expects a testing.T instance as the first
// argument.
func MustJSONEncodeBytes(t *testing.T, b []byte) *apiextensionsv1.JSON {
	data, err := json.Marshal(string(b))
	if err != nil {
		t.Errorf("could not encode bytes to json: %s", err)
	}
	return &apiextensionsv1.JSON{Raw: data}
}
